// hio.cpp : implementation of the CHiO class
//

#include "stdafx.h"
#include "hio.h"
#include "math.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
///////////////////////////////////////////////
// some description data

#define GtR 0.01745329251994f // degrees to radians
// defining some geometry for the logo

#define RO	1.0f	// outer radius
#define D	0.2f	// stroke thickness
#define H	0.2f	// symbol "height", z-extension

#define BOTTOM -0.26f	// bottom of lines
#define LTOP	0.61f	// top left line
#define RTOP	0.25f	// top right line

#define SHEARV	20.0f	// degrees shear


/////////////////////////////////////////////////////////////////////////////
// CHiO

IMPLEMENT_DYNCREATE(CHiO, CObject)


/////////////////////////////////////////////////////////////////////////////
// CHiO construction/destruction
CHiO::CHiO()
{
	m_nP=40;
	m_Display_List=-1;
}

CHiO::CHiO(UINT nPnts)
{	
	m_Display_List=-1;
	m_nP=nPnts;
}

CHiO::~CHiO()
{
}
/////////////////////////////////////////////////////////////////////////////
// CHiO diagnostics

#ifdef _DEBUG
void CHiO::AssertValid() const
{
	CObject::AssertValid();
}

void CHiO::Dump(CDumpContext& dc) const
{
	CObject::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
void CHiO::DrawLogo()
{
	if(m_Display_List!=-1)
		glCallList(m_Display_List);
	else
		Initialize(m_nP);
}

void CHiO::SetPrecision(int n)
{
	m_nP=max(n,4);
	Initialize(m_nP);
}


void CHiO::Initialize(UINT n)
{
	
	GLfloat *m_Co=NULL;// list for outer circlesegment
	GLfloat *m_Ci=NULL;// list for inner circle segment
	GLfloat m_LL[12];// list for left line
	GLfloat m_RL[12];// list for right line
	
	// degrees to radians
	GLfloat rSHEARV=SHEARV*GtR;
	
	// derive geometry from key-constants

	// angles
	GLfloat rVO1,rVO2,rVI1,rVI2,DVO,DVI;

	// for instance R(ight line)T(op)L(eft)
	GLfloat LTL,LBL,RTL,RBL;

	// stroke as measured with VSHEAR
	GLfloat sD=D/(GLfloat)cos(rSHEARV);

	// left line low-left
	// same as -fabs(BOTTOM), since BOTTOM is negative
	LBL=BOTTOM*(GLfloat)tan(rSHEARV)-1.5f*sD;
	// right line low-left
	RBL=LBL+2.0f*sD;
	// left line top-left
	LTL=LTOP*(GLfloat)tan(rSHEARV)-1.5f*sD;
	// right line top-left
	RTL=RTOP*(GLfloat)tan(rSHEARV)+0.5f*sD;


	// set up the lines
	m_LL[0]=LTL;	m_LL[1]=LTOP;	m_LL[2]=0.0f;
	m_LL[3]=LBL;   m_LL[4]=BOTTOM;	m_LL[5]=0.0f;
	m_LL[6]=LBL+sD; m_LL[7]=BOTTOM;	m_LL[8]=0.0f;
	m_LL[9]=LTL+sD;	m_LL[10]=LTOP;	m_LL[11]=0.0f;

	m_RL[0]=RTL;	m_RL[1]=RTOP;	m_RL[2]=0.0f;
	m_RL[3]=RBL;	m_RL[4]=BOTTOM;	m_RL[5]=0.0f;
	m_RL[6]=RBL+sD; m_RL[7]=BOTTOM;	m_RL[8]=0.0f;
	m_RL[9]=RTL+sD;	m_RL[10]=RTOP;	m_RL[11]=0.0f;
	
	
	// Then we consentrate on the circle-arcs
	
	// set params for outer circle
	// startangle outer circle
	rVO1=(GLfloat)atan2(
		RO*(GLfloat)sin((90*GtR-rSHEARV)),
		RO*(GLfloat)cos((90*GtR-rSHEARV))-0.5*sD);
	// stoppangle outer circle
	rVO2=(GLfloat)atan2(
		RO*(GLfloat)sin((90*GtR-rSHEARV)),
		RO*(GLfloat)cos((90*GtR-rSHEARV))+0.5*sD);
	// sector
	DVO=360.0f*GtR-(GLfloat)fabs(rVO1-rVO2);

	// set params for inner circle
	// startangle inner circle
	rVI1=(GLfloat)atan2(
		(RO-D)*(GLfloat)sin((90-SHEARV)*GtR),
		(RO-D)*(GLfloat)cos((90-SHEARV)*GtR)-0.5f*sD);
	// stopangle inner circle
	rVI2=(GLfloat)atan2(
		(RO-D)*(GLfloat)sin((90-SHEARV)*GtR),
		(RO-D)*(GLfloat)cos((90-SHEARV)*GtR)+0.5f*sD);
	DVI=360.0f*GtR-(GLfloat)fabs(rVI1-rVI2);

	// establish a list of points,nPnts, for
	// outer and inner pointlist

	// memory will be freed in end of function

	HANDLE hHeap=GetProcessHeap();
	m_Co=(GLfloat*)HeapAlloc(hHeap,0,3*n*sizeof(GLfloat));
	m_Ci=(GLfloat*)HeapAlloc(hHeap,0,3*n*sizeof(GLfloat));
	m_nP=n;

	// make lists, same noof pnts, m_nP, in each
	UINT ix;
	GLfloat v,dv;
	GLfloat r;
	// outer circle segment
	v=rVO1;
	r=RO;
	m_Co[0]=r*(GLfloat)cos( v);
	m_Co[1]=r*(GLfloat)sin( v);
	m_Co[2]=(GLfloat)0.0f;
	dv=DVO/(m_nP-1);
	for(ix=1;ix<m_nP;ix++)
	{
		v+=dv;
		m_Co[ix*3+0]=r*(GLfloat)cos( v);
		m_Co[ix*3+1]=r*(GLfloat)sin( v);
		m_Co[ix*3+2]=(GLfloat)0.0f;

	}
	// inner circle segment
	v=rVI1;
	r=RO-D;
	m_Ci[0]=r*(GLfloat)cos( v);
	m_Ci[1]=r*(GLfloat)sin( v);
	m_Ci[2]=(GLfloat)0.0f;
	dv=DVI/(m_nP-1);
	for(ix=1;ix<m_nP;ix++)
	{
		v+=dv;
		m_Ci[ix*3+0]=r*(GLfloat)cos( v);
		m_Ci[ix*3+1]=r*(GLfloat)sin( v);
		m_Ci[ix*3+2]=(GLfloat)0.0f;

	}

	////////////////////////////////////////////
	// make display list
	if(m_Display_List!=-1)
		glDeleteLists(m_Display_List,1);
	m_Display_List=glGenLists(1);
	glNewList(m_Display_List,GL_COMPILE_AND_EXECUTE);

	
	// draw circle-polygons top (z-wise)
	glBegin(GL_TRIANGLE_STRIP);
	glNormal3f(0.0f,0.0f,1.0f);
	for(ix=0;ix<m_nP;ix++)
		{
			glVertex3f(m_Co[3*ix+0],m_Co[3*ix+1],m_Co[3*ix+2]);
			glVertex3f(m_Ci[3*ix+0],m_Ci[3*ix+1],m_Ci[3*ix+2]);
		}
	glEnd();
	// draw circle-polygons bottom
	glBegin(GL_TRIANGLE_STRIP);
	glNormal3f(0.0f,0.0f,-1.0f);
		for(ix=0;ix<m_nP;ix++)
		{
			glVertex3f(m_Co[3*ix+0],m_Co[3*ix+1],m_Co[3*ix+2]-H);
			glVertex3f(m_Ci[3*ix+0],m_Ci[3*ix+1],m_Ci[3*ix+2]-H);
		}
	glEnd();

	// draw circle-polygons inner
	glBegin(GL_TRIANGLE_STRIP);
		for(ix=0;ix<m_nP;ix++)
		{
			// normal twds origo
			glNormal3f(-m_Ci[3*ix+0],-m_Ci[3*ix+1],-m_Ci[3*ix+2]);
			glVertex3f(m_Ci[3*ix+0],m_Ci[3*ix+1],m_Ci[3*ix+2]);
			glVertex3f(m_Ci[3*ix+0],m_Ci[3*ix+1],m_Ci[3*ix+2]-H);
		}
	glEnd();

	// draw circle-polygons outer
	glBegin(GL_TRIANGLE_STRIP);
		for(ix=0;ix<m_nP;ix++)
		{
			// normal as radius prolongation
			glNormal3f(m_Co[3*ix+0],m_Co[3*ix+1],m_Co[3*ix+2]);
			glVertex3f(m_Co[3*ix+0],m_Co[3*ix+1],m_Co[3*ix+2]);
			glVertex3f(m_Co[3*ix+0],m_Co[3*ix+1],m_Co[3*ix+2]-H);
		}
	glEnd();

	// draw circle-plygon start-end
	glBegin(GL_POLYGON);
		glVertex3f(m_Co[0],m_Co[1],m_Co[2]);
		glVertex3f(m_Ci[0],m_Ci[1],m_Ci[2]);
		glVertex3f(m_Ci[0],m_Ci[1],m_Ci[2]-H);
		glVertex3f(m_Co[0],m_Co[1],m_Co[2]-H);
	glEnd();

	// draw circle-plygon stop-end
	glBegin(GL_POLYGON);
		glVertex3f(m_Co[3*(m_nP-1)],m_Co[3*(m_nP-1)+1],m_Co[3*(m_nP-1)+2]);
		glVertex3f(m_Ci[3*(m_nP-1)],m_Ci[3*(m_nP-1)+1],m_Ci[3*(m_nP-1)+2]);
		glVertex3f(m_Ci[3*(m_nP-1)],m_Ci[3*(m_nP-1)+1],m_Ci[3*(m_nP-1)+2]-H);
		glVertex3f(m_Co[3*(m_nP-1)],m_Co[3*(m_nP-1)+1],m_Co[3*(m_nP-1)+2]-H);
	glEnd();
	
	// leftmost stroke
	// draw lefmost stroke top
	glBegin(GL_POLYGON);
		glNormal3f(0.0f,0.0f,1.0f);
		glVertex3f(m_LL[0],m_LL[1],m_LL[2]);
		glVertex3f(m_LL[3],m_LL[4],m_LL[5]);
		glVertex3f(m_LL[6],m_LL[7],m_LL[8]);
		glVertex3f(m_LL[9],m_LL[10],m_LL[11]);
	glEnd();
	
	// draw lefmost stroke bottom
	glBegin(GL_POLYGON);
		glNormal3f(0.0f,0.0f,-1.0f);
		glVertex3f(m_LL[0],m_LL[1],m_LL[2]-H);
		glVertex3f(m_LL[3],m_LL[4],m_LL[5]-H);
		glVertex3f(m_LL[6],m_LL[7],m_LL[8]-H);
		glVertex3f(m_LL[9],m_LL[10],m_LL[11]-H);
	glEnd();

	// draw lefmost stroke left
	glBegin(GL_POLYGON);
		glNormal3f(-1.0f,0.0f,0.0f); // approx
		glVertex3f(m_LL[0],m_LL[1],m_LL[2]);
		glVertex3f(m_LL[0],m_LL[1],m_LL[2]-H);
		glVertex3f(m_LL[3],m_LL[4],m_LL[5]-H);
		glVertex3f(m_LL[3],m_LL[4],m_LL[5]);
	glEnd();

	// draw lefmost stroke right
	glBegin(GL_POLYGON);
		glNormal3f(1.0f,0.0f,0.0f); // approx
		glVertex3f(m_LL[6],m_LL[7],m_LL[8]);
		glVertex3f(m_LL[6],m_LL[7],m_LL[8]-H);
		glVertex3f(m_LL[9],m_LL[10],m_LL[11]-H);
		glVertex3f(m_LL[9],m_LL[10],m_LL[11]);
	glEnd();

	// draw lefmost stroke upper
	glBegin(GL_POLYGON);
		glNormal3f(0.0f,1.0f,0.0f); 
		glVertex3f(m_LL[0],m_LL[1],m_LL[2]);
		glVertex3f(m_LL[9],m_LL[10],m_LL[11]);
		glVertex3f(m_LL[9],m_LL[10],m_LL[11]-H);
		glVertex3f(m_LL[0],m_LL[1],m_LL[2]-H);
	glEnd();


	// draw lefmost stroke lower
	glBegin(GL_POLYGON);
		glNormal3f(0.0f,-1.0f,0.0f); 
		glVertex3f(m_LL[3],m_LL[4],m_LL[5]);
		glVertex3f(m_LL[6],m_LL[7],m_LL[8]);
		glVertex3f(m_LL[6],m_LL[7],m_LL[8]-H);
		glVertex3f(m_LL[3],m_LL[4],m_LL[5]-H);
	glEnd();

	// rightmost stroke
	// draw rightmost stroke top
	glBegin(GL_POLYGON);
		glNormal3f(0.0f,0.0f,1.0f);
		glVertex3f(m_RL[0],m_RL[1],m_RL[2]);
		glVertex3f(m_RL[3],m_RL[4],m_RL[5]);
		glVertex3f(m_RL[6],m_RL[7],m_RL[8]);
		glVertex3f(m_RL[9],m_RL[10],m_RL[11]);
	glEnd();
	
	// draw rightmost stroke bottom
	glBegin(GL_POLYGON);
		glNormal3f(0.0f,0.0f,-1.0f);
		glVertex3f(m_RL[0],m_RL[1],m_RL[2]-H);
		glVertex3f(m_RL[3],m_RL[4],m_RL[5]-H);
		glVertex3f(m_RL[6],m_RL[7],m_RL[8]-H);
		glVertex3f(m_RL[9],m_RL[10],m_RL[11]-H);
	glEnd();

	// draw rightmost stroke left
	glBegin(GL_POLYGON);
		glNormal3f(-1.0f,0.0f,0.0f); // approx
		glVertex3f(m_RL[0],m_RL[1],m_RL[2]);
		glVertex3f(m_RL[0],m_RL[1],m_RL[2]-H);
		glVertex3f(m_RL[3],m_RL[4],m_RL[5]-H);
		glVertex3f(m_RL[3],m_RL[4],m_RL[5]);
	glEnd();

	// draw rightmost stroke right
	glBegin(GL_POLYGON);
		glNormal3f(1.0f,0.0f,0.0f); // approx
		glVertex3f(m_RL[6],m_RL[7],m_RL[8]);
		glVertex3f(m_RL[6],m_RL[7],m_RL[8]-H);
		glVertex3f(m_RL[9],m_RL[10],m_RL[11]-H);
		glVertex3f(m_RL[9],m_RL[10],m_RL[11]);
	glEnd();

	// draw rightmost stroke upper
	glBegin(GL_POLYGON);
		glNormal3f(0.0f,1.0f,0.0f); 
		glVertex3f(m_RL[0],m_RL[1],m_RL[2]);
		glVertex3f(m_RL[9],m_RL[10],m_RL[11]);
		glVertex3f(m_RL[9],m_RL[10],m_RL[11]-H);
		glVertex3f(m_RL[0],m_RL[1],m_RL[2]-H);
	glEnd();


	// draw rightmost stroke lower
	glBegin(GL_POLYGON);
		glNormal3f(0.0f,-1.0f,0.0f); 
		glVertex3f(m_RL[3],m_RL[4],m_RL[5]);
		glVertex3f(m_RL[6],m_RL[7],m_RL[8]);
		glVertex3f(m_RL[6],m_RL[7],m_RL[8]-H);
		glVertex3f(m_RL[3],m_RL[4],m_RL[5]-H);
	glEnd();


	// here is the end of make displaylist
	///////////////////////////////////////////
	glEndList();
	
	// release memory
	HeapFree(hHeap,0,m_Co);
	HeapFree(hHeap,0,m_Ci);


}

