///////////////////////////////////////////////
// floatView.cpp : implementation of the CFloatView class
//
// loads a bitmap file and starts an OpenGL window on top of
// bitmap. See OnShow()  for details
// can only handle 8-bits bmps

#include "stdafx.h"
#include "float.h"

#include "floatDoc.h"
#include "floatView.h"
#include "mainfrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CFloatView

IMPLEMENT_DYNCREATE(CFloatView, CView)

BEGIN_MESSAGE_MAP(CFloatView, CView)
	//{{AFX_MSG_MAP(CFloatView)
	ON_COMMAND(ID_LOAD_BM, OnLoadBm)
	ON_COMMAND(ID_SHOW, OnShow)
	ON_UPDATE_COMMAND_UI(ID_SHOW, OnUpdateShow)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFloatView construction/destruction

CFloatView::CFloatView()
{
	m_pOGL=NULL;
}

CFloatView::~CFloatView()
{
	if(m_pOGL!=NULL)
		delete m_pOGL;
}

BOOL CFloatView::PreCreateWindow(CREATESTRUCT& cs)
{
	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CFloatView drawing

void CFloatView::OnDraw(CDC* pDC)
{
	CFloatDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// draw background bitmap if it exists
	if((m_Back.GetWidth()>16)&&(m_Back.GetHeight()>16))
		m_Back.Draw(pDC,0,0);
}

/////////////////////////////////////////////////////////////////////////////
// CFloatView diagnostics

#ifdef _DEBUG
void CFloatView::AssertValid() const
{
	CView::AssertValid();
}

void CFloatView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CFloatDoc* CFloatView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CFloatDoc)));
	return (CFloatDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CFloatView message handlers

void CFloatView::OnLoadBm() 
{
	// Load a background bitmap
	// prompts for a file of type bmp or dib, rudimentary code
	//CFileDialog FD(TRUE,".bmp","*.bmp",NULL,"Bitmap Files (*.bmp)||");
	CFileDialog FD(TRUE,".bmp","*.bmp",NULL,"Device Independant Bitmaps (*.bmp)|*.bmp|Device Independant Bitmaps (*.dib)|*.dib||");
	if(!FD.DoModal())
		return;
	CString filename=FD.GetPathName();
	if(filename.GetLength()<4)
		return;
	m_Back.Load(filename);	
	ASSERT((m_Back.GetWidth()>0)&&(m_Back.GetHeight()>0));

	// hide opengl-window is it active
	if(m_pOGL!=NULL)
		m_pOGL->ShowWindow(SW_HIDE);

	// ...and size window accordingly
	// but not smaller than 100
	int dx=max(m_Back.GetWidth(),100);
	int dy=max(m_Back.GetHeight(),100);
	pMain->SetWindowPos(NULL,0,0,
					dx+2*GetSystemMetrics(SM_CXEDGE),
					dy+2*GetSystemMetrics(SM_CYEDGE)+
					GetSystemMetrics(SM_CYCAPTION)+
					GetSystemMetrics(SM_CYMENU),
					SWP_NOZORDER|SWP_NOMOVE);
}

void CFloatView::OnShow() 
{
	// will show the opengl-window

	// if it does not exist, we make it
	if(m_pOGL==NULL)
		m_pOGL=new CMyWin(this);

	// assume it is ok
	ASSERT(m_pOGL!=NULL);
	// show the opengl-window
	if(!m_pOGL->IsWindowVisible())
	{
		if((m_Back.GetWidth()<32)||(m_Back.GetHeight()<32))
		{
			// no bitmap to overlay,
			// select biggest square in clientarea as OpenGL window
			CRect cR;
			GetClientRect(&cR);
			int d=min(cR.Width(),cR.Height());
			CRect R((cR.Width()-d)/2,(cR.Height()-d)/2,
					(cR.Width()+d)/2,(cR.Height()+d)/2);
			m_pOGL->SetWindowPos(NULL,R.left,R.top,R.Width(),R.Height(),
									SWP_NOZORDER);
			m_pOGL->ShowWindow(SW_SHOW);

		}
		else
		{
			// prepare bitmaps and OGLimage
		
			// decide wich part of the window we will
			// open the OpenGL-window in
			// we ant to operate with a square 2n size window
			int d=1;
			while(d < min(m_Back.GetWidth(),m_Back.GetHeight()))
				d*=2;
			d/=2; // size of square
			
			// want it centered in backimage
			// assume backimage in clientareas upper left corner
			CRect R((m_Back.GetWidth()-d)/2,(m_Back.GetHeight()-d)/2,
				(m_Back.GetWidth()+d)/2,(m_Back.GetHeight()+d)/2);
			
			CDIB dib;
			CDIBPal pal;

			pal.Create(&m_Back);
			dib.Create(R.Width(),R.Height(),(CPalette*)&pal);
			m_Back.CopyBits(&dib,0,0,R.Width(),R.Height(),R.left,R.top);
			m_pOGL->m_OGLIMAGE.Translate(dib,TRANS_DIY);
			
			m_pOGL->SetWindowPos(NULL,R.left,R.top,R.Width(),R.Height(),
									SWP_NOZORDER);
			m_pOGL->ShowWindow(SW_SHOW);
		}
		
	}
	else
		m_pOGL->ShowWindow(SW_HIDE);
		

}

void CFloatView::OnUpdateShow(CCmdUI* pCmdUI) 
{
	// mark if showing
	if(m_pOGL!=NULL)
		if(m_pOGL->IsWindowVisible())
			pCmdUI->SetCheck();
	
}
